@@ -27,15 +27,12 @@
 #include "rsvg-css.h"
 
 #include <glib.h>
-#include <math.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
-#include <stdio.h>
-
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif  /*  M_PI  */
+#include <math.h>
 
 #define POINTS_PER_INCH (72.0)
 #define CM_PER_INCH     (2.54)
@@ -56,31 +53,27 @@ gboolean
 rsvg_css_parse_vbox (const char * vbox, double * x, double * y,
 					 double * w, double * h)
 {
-	/* TODO: make me cleaner and more efficient */
-	char *ptr, *tok;
-	char *str = g_strdup (vbox);
-	gboolean has_vbox = FALSE;
-	
-	tok = strtok_r (str, ", \t", &ptr);
-	if (tok != NULL) {
-		*x = g_ascii_strtod (tok, NULL);
-		tok = strtok_r (NULL, ", \t", &ptr);
-		if (tok != NULL) {
-			*y = g_ascii_strtod (tok, NULL);
-			tok = strtok_r (NULL, ", \t", &ptr);
-			if (tok != NULL) {
-				*w = g_ascii_strtod (tok, NULL);
-				tok = strtok_r (NULL, ", \t", &ptr);
-				if (tok != NULL) {
-					*h = g_ascii_strtod (tok, NULL);
-					has_vbox = TRUE;
-				}
-			}
-		}
+	gchar ** list;
+	guint list_len;
+
+	list = rsvg_css_parse_list(vbox, &list_len);
+
+	if(!list)
+		return FALSE;
+	else if(list_len != 4) {
+		g_strfreev(list);
+		return FALSE;
+	} 
+	else {
+		/* TODO: error checking */
+		*x = g_ascii_strtod(list[0], NULL);
+		*y = g_ascii_strtod(list[1], NULL);
+		*w = g_ascii_strtod(list[2], NULL);
+		*h = g_ascii_strtod(list[3], NULL);
+
+		g_strfreev(list);
+		return TRUE;
 	}
-	g_free (str);
-	
-	return has_vbox;
 }
 
 /**
@@ -354,7 +347,7 @@ rsvg_css_parse_color (const char *str)
 					{ "fuchsia",              PACK_RGB (255,0,255) },
 					{ "gainsboro",            PACK_RGB (220,220,220) },
 					{ "ghostwhite",           PACK_RGB (248,248,255) },
-					{ "gold",                 PACK_RGB (215,215,0) },
+					{ "gold",                 PACK_RGB (255,215,0) },
 					{ "goldenrod",            PACK_RGB (218,165,32) },
 					{ "gray",                 PACK_RGB (128,128,128) },
 					{ "grey",                 PACK_RGB (128,128,128) },
@@ -510,7 +503,7 @@ rsvg_css_parse_angle (const char * str)
 	if (end_ptr)
 		{
 			if (!strcmp(end_ptr, "rad"))
-				return degrees * 180. / M_PI;
+				return degrees * 180. / G_PI;
 			else if (!strcmp(end_ptr, "grad"))
 				return degrees * 360. / 400.;
 		}
@@ -532,19 +525,19 @@ rsvg_css_parse_angle (const char * str)
 double
 rsvg_css_parse_frequency (const char * str)
 {
-	double hz;
+	double f_hz;
 	char *end_ptr;
 	
-	hz = g_ascii_strtod (str, &end_ptr);
+	f_hz = g_ascii_strtod (str, &end_ptr);
 	
 	/* todo: error condition - figure out how to best represent it */
-	if ((hz == -HUGE_VAL || hz == HUGE_VAL) && (ERANGE == errno))
+	if ((f_hz == -HUGE_VAL || f_hz == HUGE_VAL) && (ERANGE == errno))
 		return 0.0;
 	
 	if (end_ptr && !strcmp(end_ptr, "kHz"))
-		return hz * 1000.;
+		return f_hz * 1000.;
 	
-	return hz;
+	return f_hz;
 }
 
 /*
@@ -675,3 +668,121 @@ rsvg_css_parse_font_family (const char * str, const char * inherit)
 	else
 		return str;
 }
+
+#if !defined(HAVE_STRTOK_R) && !GLIB_CHECK_VERSION(2, 3, 2)
+
+static char *
+strtok_r(char *s, const char *delim, char **last)
+{
+	char *p;
+
+	if (s == NULL)
+		s = *last;
+
+	if (s == NULL)
+		return NULL;
+
+	while (*s && strchr (delim, *s))
+		s++;
+
+	if (*s == '\0') {
+		*last = NULL;
+		return NULL;
+	}
+
+	p = s;
+	while (*p && !strchr (delim, *p))
+		p++;
+
+	if (*p == '\0')
+		*last = NULL;
+	else {
+		*p = '\0';
+		p++;
+		*last = p;
+	}
+	
+	return s;
+}
+
+#endif /* !HAVE_STRTOK_R */
+
+gchar **
+rsvg_css_parse_list(const char * in_str, guint * out_list_len)
+{
+#if GLIB_CHECK_VERSION(2, 3, 2)
+
+	gchar ** string_array;
+	guint n;
+
+	/* this may fix bug #113538 */
+
+	string_array = g_strsplit_set(in_str, ", \t", -1);
+
+	for(n = 0; string_array[n] != NULL; n++)
+		;
+
+	*out_list_len = n;
+	return string_array;
+
+#else
+
+	char *ptr, *tok;
+	char *str;
+
+	guint n = 0;
+	GSList * string_list = NULL;
+	gchar ** string_array = NULL;
+
+	str = g_strdup (in_str);
+    tok = strtok_r (str, ", \t", &ptr);
+	if (tok != NULL) {
+		string_list = g_slist_prepend(string_list, g_strdup(tok));
+		n++;
+
+		while((tok = strtok_r (NULL, ", \t", &ptr)) != NULL) {
+			string_list = g_slist_prepend(string_list, g_strdup(tok));
+			n++;
+		}
+	}
+	g_free (str);
+
+	if(out_list_len)
+		*out_list_len = n;
+
+	if (string_list) {
+		GSList *slist;
+
+		string_array = g_new(gchar *, n + 1);
+
+		string_array[n--] = NULL;
+		for (slist = string_list; slist; slist = slist->next)
+			string_array[n--] = (gchar *)slist->data;
+
+		g_slist_free (string_list);		
+	}
+
+	return string_array;
+
+#endif
+}
+
+void 
+rsvg_css_parse_number_optional_number(const char * str, 
+									  double *x, double *y)
+{
+  char *endptr;
+
+  /* TODO: some error checking */
+
+  *x = g_ascii_strtod(str, &endptr);
+
+  if(endptr && *endptr != '\0')
+    while(g_ascii_isspace(*endptr) && *endptr)
+      endptr++;
+
+  if(endptr && *endptr)
+    *y = g_ascii_strtod(endptr, NULL);
+  else
+    *y = *x;
+}
