@@ -41,7 +41,7 @@ rsvg_viewport_percentage (gdouble width, gdouble height)
 void
 rsvg_state_init (RsvgState *state)
 {
-	memset (state, 0, sizeof (*state));
+	memset (state, 0, sizeof (RsvgState));
 	
 	art_affine_identity (state->affine);
 	
@@ -54,6 +54,7 @@ rsvg_state_init (RsvgState *state)
 	state->cap = ART_PATH_STROKE_CAP_BUTT;
 	state->join = ART_PATH_STROKE_JOIN_MITER;
 	state->stop_opacity = 0xff;
+	state->fill_rule = FILL_RULE_NONZERO;
 	
 	state->font_family  = g_strdup ("Times New Roman");
 	state->font_size    = 12.0;
@@ -62,6 +63,7 @@ rsvg_state_init (RsvgState *state)
 	state->font_weight  = PANGO_WEIGHT_NORMAL;
 	state->font_stretch = PANGO_STRETCH_NORMAL;
 	state->text_dir     = PANGO_DIRECTION_LTR;
+	state->text_anchor  = TEXT_ANCHOR_START;
 	state->visible      = TRUE;
 }
 
@@ -72,10 +74,12 @@ rsvg_state_clone (RsvgState *dst, const RsvgState *src)
 	
 	*dst = *src;
 	dst->font_family = g_strdup (src->font_family);
+	dst->lang = g_strdup (src->lang);
 	rsvg_paint_server_ref (dst->fill);
 	rsvg_paint_server_ref (dst->stroke);
 	dst->save_pixbuf = NULL;
-	
+	dst->opacity = 0xff;	
+
 	if (src->dash.n_dash > 0)
 		{
 			dst->dash.dash = g_new (gdouble, src->dash.n_dash);
@@ -88,6 +92,7 @@ void
 rsvg_state_finalize (RsvgState *state)
 {
 	g_free (state->font_family);
+	g_free (state->lang);
 	rsvg_paint_server_unref (state->fill);
 	rsvg_paint_server_unref (state->stroke);
 	
@@ -132,6 +137,14 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		{
 			state->fill_opacity = rsvg_css_parse_opacity (str + arg_off);
 		}
+	else if (rsvg_css_param_match (str, "fill-rule"))
+		{
+			if (!strcmp (str + arg_off, "nonzero"))
+				state->fill_rule = FILL_RULE_NONZERO;
+			else if (!strcmp (str + arg_off, "evenodd"))
+				state->fill_rule = FILL_RULE_EVENODD;
+			/*else inherit*/
+		}
 	else if (rsvg_css_param_match (str, "stroke"))
 		{
 			rsvg_paint_server_unref (state->stroke);
@@ -179,6 +192,12 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			g_free (state->font_family);
 			state->font_family = save;
 		}
+	else if (rsvg_css_param_match (str, "xml:lang"))
+		{
+			char * save = g_strdup (str + arg_off);
+			g_free (state->lang);
+			state->lang = save;
+		}
 	else if (rsvg_css_param_match (str, "font-style"))
 		{
 			state->font_style = rsvg_css_parse_font_style (str + arg_off, parent_state->font_style);
@@ -224,6 +243,20 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 			else
 				state->text_dir = PANGO_DIRECTION_LTR;
 		}
+	else if (rsvg_css_param_match (str, "text-anchor"))
+		{
+			if (!strcmp (str + arg_off, "inherit"))
+				state->text_anchor = parent_state->text_anchor;
+			else 
+				{
+					if (strstr (str + arg_off, "start"))
+						state->text_anchor = TEXT_ANCHOR_START;
+					else if (strstr (str + arg_off, "middle"))
+						state->text_anchor = TEXT_ANCHOR_MIDDLE;
+					else if (strstr (str + arg_off, "end") )
+						state->text_anchor = TEXT_ANCHOR_END;
+				}
+		}
 	else if (rsvg_css_param_match (str, "stop-color"))
 		{
 			state->stop_color = rsvg_css_parse_color (str + arg_off);
@@ -302,6 +335,8 @@ rsvg_is_style_arg(const char *str)
 			g_hash_table_insert (styles, "display",           GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "fill",              GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "fill-opacity",      GINT_TO_POINTER (TRUE));
+			g_hash_table_insert (styles, "fill-rule",         GINT_TO_POINTER 
+(TRUE));
 			g_hash_table_insert (styles, "font-family",       GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "font-size",         GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "font-stretch",      GINT_TO_POINTER (TRUE));
@@ -319,9 +354,11 @@ rsvg_is_style_arg(const char *str)
 			g_hash_table_insert (styles, "stroke-miterlimit", GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "stroke-opacity",    GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "stroke-width",      GINT_TO_POINTER (TRUE));
+			g_hash_table_insert (styles, "text-anchor",       GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "text-decoration",   GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "visibility",        GINT_TO_POINTER (TRUE));
 			g_hash_table_insert (styles, "writing-mode",      GINT_TO_POINTER (TRUE));
+			g_hash_table_insert (styles, "xml:lang",          GINT_TO_POINTER (TRUE));
 		}
 	
 	/* this will default to 0 (FALSE) on a failed lookup */
@@ -383,7 +420,7 @@ rsvg_css_define_style (RsvgHandle *ctx, const char * style_name, const char * st
 
 #ifdef HAVE_LIBCROCO
 
-#include <libcroco.h>
+#include <libcroco/libcroco.h>
 
 typedef struct _CSSUserData
 {
@@ -436,7 +473,8 @@ ccss_end_selector (CRDocHandler *a_handler,
 }
 
 static void
-ccss_property (CRDocHandler *a_handler, GString *a_name, CRTerm *a_expr)
+ccss_property (CRDocHandler *a_handler, GString *a_name, 
+			   CRTerm *a_expr, gboolean a_important)
 {
 	CSSUserData * user_data;
 	char * expr = NULL;
@@ -445,7 +483,7 @@ ccss_property (CRDocHandler *a_handler, GString *a_name, CRTerm *a_expr)
 
 	user_data = (CSSUserData *)a_handler->app_data;
 
-	if (a_name && a_name->str && a_expr)
+	if (a_name && a_name->str && a_expr && user_data->def)
 		{
 			g_string_append_len (user_data->def, a_name->str, a_name->len);
 			g_string_append (user_data->def, ": ");
@@ -527,8 +565,6 @@ rsvg_real_parse_cssbuffer (RsvgHandle *ctx, const char * buff, size_t buflen)
 
 #else /* !HAVE_LIBCROCO */
 
-#warning Building without libcroco support. Will experience sub-optimal CSS parsing.
-
 static void
 rsvg_real_parse_cssbuffer (RsvgHandle *ctx, const char * buff, size_t buflen)
 {
